'''
Created on Jun 11, 2015

@author: will
'''
#Imports
import termios, fcntl, sys, os
from time import sleep
sys.path.append(os.path.join(os.path.dirname(__file__), ".."));
import src
import numpy
import RPi.GPIO as GPIO

from managers.sensor_manager import SensorManager
from managers.setup_motor_manager import SetupMotorManager



#Variables
state = 0
error = 0
key = 0
calibrationMode = 0

# direction data
GripperTopDirection = 0     # Large gear turns Clockwise
GripperBotDirection = 0 

GimbalPitchAxis = 0         # Is the axis inverted
GimbalTopDirection = 0      # Unwinding
GimbalBotDirection = 0

GimbalYawAxis = 0
GimbalLeftDirection = 0
GimbalRightDirection = 0

LeadScrewDirection = 0      # Extending

# pot data
GimbalTopMax = 0            # Fully released
GimbalTopMin = 0            # Fully tightened
GimbalBotMax = 0            
GimbalBotMin = 0 
GimbalLeftMax = 0            
GimbalLeftMin = 0  
GimbalRightMax = 0            
GimbalRightMin = 0 

GripperTopOpen = 0
GripperBotOpen = 0          

GPIO.cleanup()
sensor_manager = SensorManager()
drive_manager = SetupMotorManager(sensor_manager)

# Functions
def clear():
    os.system('cls' if os.name == 'nt' else 'clear')

def getKey():
    c = 0  
    i = 0
    # The recommended minimum is 1000
    while(c == 0 and i < 1200):
        try:
            c = sys.stdin.read(1)
            print "Got character", repr(c)
        except IOError: pass
        i = i + 1
    return c

# converts a string to an array removes the last char in the string and returns the new string
def backspace(aString):
    msg = numpy.chararray([100])
    msg.fill('\0')
    i = 0
    for c in aString:
        try:
            msg[i] = c
            i = i + 1
        except: pass
    aString = ""
    n = 0
    i = i - 1 # decrease size of new string by one
    while n < i:
        aString = aString + msg[n]
        n = n + 1
    return aString

#Main code
fd = sys.stdin.fileno()

oldterm = termios.tcgetattr(fd)
newattr = termios.tcgetattr(fd)
newattr[3] = newattr[3] & ~termios.ICANON & ~termios.ECHO
termios.tcsetattr(fd, termios.TCSANOW, newattr)

oldflags = fcntl.fcntl(fd, fcntl.F_GETFL)
fcntl.fcntl(fd, fcntl.F_SETFL, oldflags | os.O_NONBLOCK)



while(state != -1):
    key = 0
    
    # State 0 handles the main menu
    if(state == 0):
        # Print menu
        clear()
        print("Welcome to the SquirrelBot setup wizard.\n\n")
        print("WARNING: This program is unstable\n\n")
        print("Main Menu")
        print("1: SquirrelBot 2.1 Calibration")
        print("2: SquirrelBot 2.1 Control Test")
        print("0: Exit")
        
        # Wait for user input
        while(state == 0):
            key = getKey()
            # User selects exit
            if(key == '0'):
                state = -1 
            if(key == '1'):
                state = 1
            if(key == '2'):
                state = 5
    # End of menu code
    
    # Check for existing configuration
    if(state == 1):
        clear()
        exists = 0
        try:
            fo = open("config.txt", "r")
            exists = 1
            fo.close()
        except : pass
        if(exists == 1):
            #TODO load config
            print("A configuration file already exists")
            print("What would you like to do?\n")
            print("0: Use existing configuration")
            print("1: Create a new configuration")
            print("2: Update motor direction configuration")
            print("3: Update gimbal range configuration")
            print("4: Update all configuration data")
            while(key == 0):
                key = getKey()
                if(key == '0'):
                    state = 0
                if(key == '1'):
                    GripperTopDirection = 0
                    GripperBotDirection = 0
                    GimbalPitchAxis = 0
                    GimbalTopDirection = 0
                    GimbalBotDirection = 0
                    GimbalYawAxis = 0
                    GimbalLeftDirection = 0
                    GimbalRightDirection = 0
                    LeadScrewDirection = 0
                    state = 2
                    calibrationMode = 1
                if(key == '2'):
                    state = 2
                if(key == '3'):
                    state = 3
                if(key == '4'):
                    state = 2
                    calibrationMode = 1
            
        else:
            print("No configuration file available")
            state = 2
            calibrationMode = 1
    # End of check
    
    # Begin Direction Calibration
    if(state == 2):
        clear()
        print("Beginning Calibration...\n")
        print("Step 1:\n")
        print("Calibrate Top Gripper:")
        print("Controls:")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(GripperTopDirection == 1):
                    GripperTopDirection = 0
                    print("Motor is now normal")
                else:
                    GripperTopDirection = 1
                    print("Motor is now reversed")
            if(key == ' '):
                drive_manager.driveClaws("TOP", GripperTopDirection)
            if(key == 0):
                drive_manager.stopClaws("TOP")
        key = 0
        drive_manager.stop()
           
        clear()
        print("Step 2:\n")
        print("Calibrate Bottom Gripper:")
        print("Controls:")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(GripperBotDirection == 1):
                    GripperBotDirection = 0
                    print("Motor is now normal")
                else:
                    GripperBotDirection = 1
                    print("Motor is now reversed")
            if(key == ' '):
                drive_manager.driveClaws("BOTTOM", GripperBotDirection)
            if(key == 0):
                drive_manager.stopClaws("BOTTOM")
        key = 0
        drive_manager.stop()
        
        clear()
        print("Step 3:\n")
        print("Calibrate Gimbal Pitch Motors: Part A\n")
        print("NOTE: Motor should be set to wind it's string and")
        print("the string should wind towards the center of the gimbal.")
        print("You should be controlling the gimbal motor that will be farthest from the tree.\n")
        print("Controls:")
        print("'/' to flip axis(use this if you are controlling the wrong motor")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(GimbalTopDirection == 1):
                    GimbalTopDirection = 0
                    print("Motor is now normal")
                else:
                    GimbalTopDirection = 1
                    print("Motor is now reversed")
            if(key == "/"):
                drive_manager.stopGimbal()
                if(GimbalPitchAxis == 1):
                    GimbalPitchAxis = 0
                    print("Axis is now normal")
                else:
                    GimbalPitchAxis = 1
                    print("Axis is now flipped")
            
            if(key == ' '):
                print("Drive")
                if(GimbalPitchAxis == 0):
                    drive_manager.driveGimbalPitch("TOP", GimbalTopDirection)
                else:
                    drive_manager.driveGimbalPitch("BOT", GimbalTopDirection)
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        clear()
        print("Step 4:\n")
        print("Calibrate Gimbal Pitch Motors: Part B\n")
        print("NOTE: Motor should be set to wind it's string and")
        print("the string should wind towards the center of the gimbal.")
        print("You should be controlling the gimbal motor that will be nearest to the tree.\n")
        print("Controls:")
        print("'/' to flip axis(use this if you are controlling the wrong motor)")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(GimbalBotDirection == 1):
                    GimbalBotDirection = 0
                    print("Motor is now normal")
                else:
                    GimbalBotDirection = 1
                    print("Motor is now reversed")
            if(key == "/"):
                drive_manager.stopGimbal()
                if(GimbalPitchAxis == 1):
                    GimbalPitchAxis = 0
                    print("Axis is now normal")
                else:
                    GimbalPitchAxis = 1
                    print("Axis is now flipped")
            
            if(key == ' '):
                if(GimbalPitchAxis == 0):
                    drive_manager.driveGimbalPitch("BOT", GimbalBotDirection)
                else:
                    drive_manager.driveGimbalPitch("TOP", GimbalBotDirection)
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        clear()
        print("Step 5:\n")
        print("Calibrate Gimbal Yaw Motors: Part A\n")
        print("NOTE: Motor should be set to wind it's string and")
        print("the string should wind towards the center of the gimbal.")
        print("You should be controlling the gimbal motor that will be to the left")
        print("if you are watching the robot climb a tree.\n")
        print("Controls:")
        print("'/' to flip axis(use this if you are controlling the wrong motor")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(GimbalLeftDirection == 1):
                    GimbalLeftDirection = 0
                    print("Motor is now normal")
                else:
                    GimbalLeftDirection = 1
                    print("Motor is now reversed")
            if(key == "/"):
                drive_manager.stopGimbal()
                if(GimbalYawAxis == 1):
                    GimbalYawAxis = 0
                    print("Axis is now normal")
                else:
                    GimbalYawAxis = 1
                    print("Axis is now flipped")
            
            if(key == ' '):
                if(GimbalYawAxis == 0):
                    drive_manager.driveGimbalYaw("LEFT", GimbalLeftDirection)
                else:
                    drive_manager.driveGimbalYaw("RIGHT", GimbalLeftDirection)
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        clear()
        print("Step 6:\n")
        print("Calibrate Gimbal Yaw Motors: Part B\n")
        print("NOTE: Motor should be set to wind it's string and")
        print("the string should wind towards the center of the gimbal.")
        print("You should be controlling the gimbal motor that will be to the right")
        print("if you are watching the robot climb a tree.\n")
        print("Controls:")
        print("'/' to flip axis(use this if you are controlling the wrong motor")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(GimbalRightDirection == 1):
                    GimbalRightDirection = 0
                    print("Motor is now normal")
                else:
                    GimbalRightDirection = 1
                    print("Motor is now reversed")
            if(key == "/"):
                drive_manager.stopGimbal()
                if(GimbalYawAxis == 1):
                    GimbalYawAxis = 0
                    print("Axis is now normal")
                else:
                    GimbalYawAxis = 1
                    print("Axis is now flipped")
            
            if(key == ' '):
                if(GimbalYawAxis == 0):
                    drive_manager.driveGimbalYaw("RIGHT", GimbalRightDirection)
                else:
                    drive_manager.driveGimbalYaw("LEFT", GimbalRightDirection)
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        clear()

        print("Step 7:\n")
        print("Calibrate Lead Screw:")
        print("Controls:")
        print("\nLead Screw should extend")
        print("SPACE to drive motor")
        print("TAB to change direction")
        print("ENTER to move to next step")
        while(key != "\n"):
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == "\t"):
                if(LeadScrewDirection == 1):
                    LeadScrewDirection = 0
                    print("Motor is now normal")
                else:
                    LeadScrewDirection = 1
                    print("Motor is now reversed")
            if(key == ' '):
                if(LeadScrewDirection):
                    drive_manager.driveLeadScrew("EXTEND")
                else:
                    drive_manager.driveLeadScrew("RETRACT")
            if(key == 0):
                drive_manager.stop()
        key = 0
        drive_manager.stop()
        
        # Choose next state
        if(calibrationMode == 1):
            state = 3
        else:
            state = 4
    # End of calibration code
    
    # Begin Range Calibration
    if(state == 3):
        #TODO
        
        # Step 1 & 2 Top Gripper Calibration
        while(key != "\n"):
            clear()
            print("Sensor Calibration...\n")
            print("Step 1:\n")
            print("Calibrate Top Gripper:")
            print("Controls:")
            print("SPACE to drive motor")
            print("ENTER to move to next step")
            pot = sensor_manager.topClawMotorPot.readValue()
            print("Pot: ", pot)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == ' '):
                drive_manager.driveClaws("TOP", GripperTopDirection)
            if(key == 0):
                drive_manager.stopClaws("TOP")
        key = 0
        drive_manager.stop()
        value = str(GripperTopOpen)
        while(key != '\n'):
            clear()
            print("Step 2:")
            print("Enter a pot value that indicates when the gripper is open")
            print("The recommended value is 500")
            print("Value: " + str(value))
            key = getKey()
            if(key == '\x7f'):
                value = backspace(value)
            if(key >= '0' and key <= '9' ):
                value = value + str(key)
        key = 0
        GripperTopOpen = str(value)

        # Step 3 & 4 Bottom Gripper Calibration
        while(key != "\n"):
            clear()
            print("Step 3:\n")
            print("Calibrate Top Gripper:")
            print("Controls:")
            print("SPACE to drive motor")
            print("ENTER to move to next step")
            pot = sensor_manager.botClawMotorPot.readValue()
            print("Pot: ", pot)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == ' '):
                drive_manager.driveClaws("BOT", GripperBotDirection)
            if(key == 0):
                drive_manager.stopClaws("BOT")
        key = 0
        drive_manager.stop()
        value = str(GripperBotOpen)
        while(key != '\n'):
            clear()
            print("Step 4:")
            print("Enter a pot value that indicates when the gripper is open")
            print("The recommended value is 500")
            print("Value: " + str(value))
            key = getKey()
            if(key == '\x7f'):
                value = backspace(value)
            if(key >= '0' and key <= '9' ):
                value = value + str(key)
        key = 0
        GripperBotOpen = str(value)
        
        # Match High and Low level naming of gimbal motors
        gimbalTop = "TOP"
        gimbalBot = "BOT"
        gimbalLeft = "LEFT"
        gimbalRight = "RIGHT"
        if(GimbalPitchAxis):
            gimbalTop = "BOT"
            gimbalBot = "TOP"
        if(GimbalYawAxis):
            gimbalLeft = "RIGHT"
            gimbalRight = "LEFT"
        
        # Setup gimbal potentiometers
        pitch1 = sensor_manager.pitchMotorPot1
        pitch2 = sensor_manager.pitchMotorPot2
        yaw1 = sensor_manager.yawMotorPot1
        yaw2 = sensor_manager.yawMotorPot2
        if(GimbalPitchAxis):
            pitch1 = sensor_manager.pitchMotorPot2
            pitch2 = sensor_manager.pitchMotorPot1
        if(GimbalYawAxis):
            yaw1 = sensor_manager.yawMotorPot2
            yaw2 = sensor_manager.yawMotorPot1
            
        # Step 5 Gimbal Top Min Calibration
        while(key != "\n"):
            clear()
            print("Step 5:\n")
            print("Calibrate Top Gimbal Min:")
            print("Controls:")
            print("'+' to wind string")
            print("'-' to unwind string")
            print("SPACE to record the value when fully unwound")
            print("ENTER to move to next step")
            pot = pitch1.readValue()
            print("Pot: ", pot)
            print("Min: ", GimbalTopMin)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '+'):
                print("F")
                drive_manager.driveGimbalPitch(gimbalTop, GimbalTopDirection)
            if(key == '-'):
                print("R")
                drive_manager.driveGimbalPitch(gimbalTop, not(GimbalTopDirection))
            if(key == ' '):
                GimbalTopMin = pot
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        while(key != "\n"):
            clear()
            print("Step 6:\n")
            print("Calibrate Bottom Gimbal Min:")
            print("Controls:")
            print("'+' to wind string")
            print("'-' to unwind string")
            print("SPACE to record the value when fully unwound")
            print("ENTER to move to next step")
            pot = pitch2.readValue()
            print("Pot: ", pot)
            print("Min: ", GimbalBotMin)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '+'):
                print("F")
                drive_manager.driveGimbalPitch(gimbalBot, GimbalBotDirection)
            if(key == '-'):
                print("R")
                drive_manager.driveGimbalPitch(gimbalBot, not(GimbalBotDirection))
            if(key == ' '):
                GimbalBotMin = pot
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        #TEST steps 7, 8: left and right min
        while(key != "\n"):
            clear()
            print("Step 7:\n")
            print("Calibrate Left Gimbal Min:")
            print("Controls:")
            print("'+' to wind string")
            print("'-' to unwind string")
            print("SPACE to record the value when fully unwound")
            print("ENTER to move to next step")
            pot = yaw1.readValue()
            print("Pot: ", pot)
            print("Min: ", GimbalLeftMin)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '+'):
                print("F")
                drive_manager.driveGimbalYaw(gimbalLeft, GimbalLeftDirection)
            if(key == '-'):
                print("R")
                drive_manager.driveGimbalYaw(gimbalLeft, not(GimbalLeftDirection))
            if(key == ' '):
                GimbalLeftMin = pot
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        while(key != "\n"):
            clear()
            print("Step 8:\n")
            print("Calibrate Right Gimbal Min:")
            print("Controls:")
            print("'+' to wind string")
            print("'-' to unwind string")
            print("SPACE to record the value when fully unwound")
            print("ENTER to move to next step")
            pot = yaw2.readValue()
            print("Pot: ", pot)
            print("Min: ", GimbalRightMin)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '+'):
                print("F")
                drive_manager.driveGimbalYaw(gimbalRight, GimbalRightDirection)
            if(key == '-'):
                print("R")
                drive_manager.driveGimbalYaw(gimbalRight, not(GimbalRightDirection))
            if(key == ' '):
                GimbalRightMin = pot
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        
        #TODO steps 9, 10 top and bot max
        
        while(key != "\n"):
            clear()
            print("Step 9:\n")
            print("Calibrate Top Gimbal Max:")
            print("Controls:")
            print("'+' to wind string")
            print("'-' to unwind string")
            print("SPACE to record the value when fully wound")
            print("ENTER to move to next step")
            pot = pitch1.readValue()
            print("Pot: ", pot)
            print("Max: ", GimbalTopMax)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '+'):
                print("F")
                drive_manager.driveGimbalPitch(gimbalTop, GimbalTopDirection)
            if(key == '-'):
                print("R")
                drive_manager.driveGimbalPitch(gimbalTop, not(GimbalTopDirection))
            if(key == ' '):
                GimbalTopMax = pot
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        clear()
        print("Please Wait...")
        print("Setting Top to min")
        drive_manager.setGimbalMotor(gimbalTop, pitch1, GimbalTopMin, GimbalTopMin, GimbalTopMax, GimbalTopDirection)
        
        while(key != "\n"):
            clear()
            print("Step 10:\n")
            print("Calibrate Bot Gimbal Max:")
            print("Controls:")
            print("'+' to wind string")
            print("'-' to unwind string")
            print("SPACE to record the value when fully wound")
            print("ENTER to move to next step")
            pot = pitch2.readValue()
            print("Pot: ", pot)
            print("Max: ", GimbalBotMax)
            key = getKey()
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '+'):
                print("F")
                drive_manager.driveGimbalPitch(gimbalBot, GimbalBotDirection)
            if(key == '-'):
                print("R")
                drive_manager.driveGimbalPitch(gimbalBot, not(GimbalBotDirection))
            if(key == ' '):
                GimbalBotMax = pot
            if(key == 0):
                drive_manager.stopGimbal()
        key = 0
        drive_manager.stop()
        clear()
        print("Please Wait...")
        print("Setting Bot to min")
        drive_manager.setGimbalMotor(gimbalBot, pitch2, GimbalBotMin, GimbalBotMin, GimbalBotMax, GimbalBotDirection)
        
        
        #TODO steps 11, 12 left and right max
        
        #TODO auto calibrate new min
        
        #TODO approximate 0, 0 
        
        #TODO tune pitch tension and angle
        
        #TODO tune yaw tension and angle


        state = 4
    # End of Calibration
    
    # Write config
    if(state == 4):
        #TODO
        print("\nPlaceholder for writing config to file")
        try:
            fo = open("config.txt", "w")
            fo.write("# Auto-generated file do not modify\n")
            fo.write("VERSION=2.1\n")
            fo.write("# Direction config\n")
            fo.writelines(("GRIPPER_TOP_DIR=", str(GripperTopDirection), "\n"))
            fo.writelines(("GRIPPER_BOT_DIR=", str(GripperBotDirection), "\n"))
            fo.writelines(("GIMBAL_PITCH_AXIS=", str(GimbalPitchAxis), "\n"))
            fo.writelines(("GIMBAL_TOP_DIR=", str(GimbalTopDirection), "\n"))
            fo.writelines(("GIMBAL_BOT_DIR=", str(GimbalBotDirection), "\n"))
            fo.writelines(("GIMBAL_YAW_AXIS=", str(GimbalYawAxis), "\n"))
            fo.writelines(("GIMBAL_LEFT_DIR=", str(GimbalLeftDirection), "\n"))
            fo.writelines(("GIMBAL_RIGHT_DIR=", str(GimbalRightDirection), "\n"))
            fo.writelines(("LEADSCREW_DIR=", str(LeadScrewDirection), "\n"))
            fo.write("# Range config")
            fo.writelines(("GRIPPER_TOP_POT=", str(GripperTopOpen), "\n"))
            fo.writelines(("GRIPPER_BOT_POT=", str(GripperBotOpen), "\n"))
            fo.close()
        except : pass
        state = 0
        calibrationMode = 0
    
    # Full robot test using keyboard controls
    if(state == 5):
        #TODO load config from file
        
        # Match High and Low level naming of gimbal motors
        gimbalTop = "TOP"
        gimbalBot = "BOT"
        gimbalLeft = "LEFT"
        gimbalRight = "RIGHT"
        if(GimbalPitchAxis):
            gimbalTop = "BOT"
            gimbalBot = "TOP"
        if(GimbalYawAxis):
            gimbalLeft = "RIGHT"
            gimbalRight = "LEFT"
        
        # Setup gimbal potentiometers
        pitch1 = sensor_manager.pitchMotorPot1
        pitch2 = sensor_manager.pitchMotorPot2
        yaw1 = sensor_manager.yawMotorPot1
        yaw2 = sensor_manager.yawMotorPot2
        if(GimbalPitchAxis):
            pitch1 = sensor_manager.pitchMotorPot2
            pitch2 = sensor_manager.pitchMotorPot1
        if(GimbalYawAxis):
            yaw1 = sensor_manager.yawMotorPot2
            yaw2 = sensor_manager.yawMotorPot1
            
        # Setup lead screw
        leadScrewExtend = "RETRACT"
        leadScrewRetract = "EXTEND"
        if(LeadScrewDirection):
            leadScrewExtend = "EXTEND"
            leadScrewRetract = "RETRACT"
        # Loop screen and input
        while(state == 5):
            clear()
            print("Full Robot Test")
            print("Controls:")
            print("Top Gripper - '9'")
            print("Bot Gripper - '3'")
            print("Gimble Pitch - '8'/'2'")
            print("Gimble Yaw - '4'/'2'")
            print("Lead Screw - '+'/'-'")
            pot = sensor_manager.topClawMotorPot.readValue()
            print(("Claw Top Pot: ", pot))
            pot = sensor_manager.botClawMotorPot.readValue()
            print(("Claw Bot Pot: ", pot))
            pot = sensor_manager.pitchMotorPot1.readValue()
            print(("Pitch Top Pot: ", pot))
            pot = sensor_manager.pitchMotorPot2.readValue()
            print(("Pitch Bot Pot: ", pot))
            pot = sensor_manager.yawMotorPot1.readValue()
            print(("Yaw Top Pot: ", pot))
            pot = sensor_manager.yawMotorPot2.readValue()
            print(("Yaw Bot Pot: ", pot))
            key = getKey()
            if(key == '\n'):
                state = 0
            if(key == '\x1b'):
                print("hit ESC")
                drive_manager.stop()
            if(key == '9'):
                drive_manager.driveClaws("TOP", GripperTopDirection)
            if(key == '3'):
                drive_manager.driveClaws("BOT", GripperTopDirection)
            if(key == '2'):
                drive_manager.driveGimbalPitch(gimbalTop, GimbalTopDirection)
                drive_manager.driveGimbalPitch(gimbalBot, not(GimbalBotDirection))
            if(key == '8'):
                drive_manager.driveGimbalPitch(gimbalTop, not(GimbalTopDirection))
                drive_manager.driveGimbalPitch(gimbalBot, GimbalBotDirection)
            if(key == '4'):
                drive_manager.driveGimbalYaw(gimbalLeft, GimbalLeftDirection)
                drive_manager.driveGimbalYaw(gimbalRight, not(GimbalRightDirection))
            if(key == '6'):
                drive_manager.driveGimbalYaw(gimbalLeft, not(GimbalLeftDirection))
                drive_manager.driveGimbalYaw(gimbalRight, GimbalRightDirection)
            if(key == '+'):
                drive_manager.driveLeadScrew(leadScrewExtend)
            if(key == '-'):
                drive_manager.driveLeadScrew(leadScrewRetract)
            
            # gimbal tuning controls    
            if(key == 'w'):
                drive_manager.driveGimbalPitch(gimbalTop, GimbalTopDirection)
            if(key == 's'):
                drive_manager.driveGimbalPitch(gimbalTop, not(GimbalTopDirection))
            if(key == 'e'):
                drive_manager.driveGimbalPitch(gimbalBot, GimbalBotDirection)
            if(key == 'd'):
                drive_manager.driveGimbalPitch(gimbalBot, not(GimbalBotDirection))
            if(key == 'q'):
                drive_manager.driveGimbalYaw(gimbalLeft, GimbalLeftDirection)
            if(key == 'a'):
                drive_manager.driveGimbalYaw(gimbalLeft, not(GimbalLeftDirection))
            if(key == 'r'):
                drive_manager.driveGimbalYaw(gimbalRight, GimbalRightDirection)
            if(key == 'f'):
                drive_manager.driveGimbalYaw(gimbalRight, not(GimbalRightDirection))
            if(key == 0):
                print("stop")
                drive_manager.stop()
                sleep(0.5)
        drive_manager.stop()
        

termios.tcsetattr(fd, termios.TCSAFLUSH, oldterm)
fcntl.fcntl(fd, fcntl.F_SETFL, oldflags)
clear()

